#include "callbacks.h"
#include "config.h"

void show_xdef_usage(void);

/*
 * Funkcja ma sprawdzi czy zosta ustawiony parametr -D | -conf_path i
 * jeli tak ustawi globaln zmienn, z ktrej korzysta nastpnie
 * funkcja odczytujca konfiguracj z plikw.
 *
 */
void read_conf_path(int argc, char **argv, configData *conf_params)
{
	int c;

	sprintf(conf_params->conf_path,"%s",CONF_DIR);
	
	while (1)
	{
		int option_index = 0;
		static struct option long_options[] = {
				  {"conf_path",1,0,'D'},
				  {0, 0, 0, 0}
		};

		c = getopt_long (argc, argv, "D:", long_options, &option_index);
		if (c == -1) break;
		else if ( c == 'D' ) strncpy(conf_params->conf_path, optarg, 256);
	}

}

/*
 * Funkcja odczytuje parametry z pliku konfiguracyjnego. Jeli zmienna
 * conf_params->conf_path jest ustawiona korzysta z niej jako rootdir dla
 * plikw z konfiguracj.
 * 
 */
void read_fileopt(configData *conf_params)
{
	char buff[256];

	/* 
	 * Konfiguracja bazy danych.
	 */
	sprintf(buff, "%s/%s", conf_params->conf_path, DB_CONF);
	search(buff, "db_name", conf_params->db_name);
	search(buff, "host", conf_params->db_host);
	search(buff, "port", conf_params->db_port);
	search(buff, "user", conf_params->db_user);
	search(buff, "passwd", conf_params->db_password);

	/*
	 *  Konfiguracja sprzedawcy.
	 */
	sprintf(buff, "%s/%s", conf_params->conf_path, SPRZEDAWCA_CONF);
	search(buff, "nazwa_firmy", conf_params->sp_nazwa_firmy);
	search(buff, "wlasciciel", conf_params->sp_wlasciciel);
	search(buff, "kod_pocztowy", conf_params->sp_kod_pocztowy);
	search(buff, "miejscowosc", conf_params->sp_miejscowosc);
	search(buff, "miejsce_wystawienia", conf_params->sp_miejsce_wystawienia);
	search(buff, "telefon", conf_params->sp_telefon);
	search(buff, "fax", conf_params->sp_fax);
	search(buff, "komorka", conf_params->sp_komorka);
	search(buff, "bank", conf_params->sp_bank);
	search(buff, "nr_konta", conf_params->sp_nr_konta);
	search(buff, "ulica", conf_params->sp_ulica);
	search(buff, "nip", conf_params->sp_nip);
	search(buff, "email", conf_params->sp_email);
	search(buff, "www", conf_params->sp_www);
	
	/*
	 * Konfiguracja xdef.
	 */
	sprintf(buff, "%s/%s", conf_params->conf_path, XDEFACTOR_CONF);
	search(buff, "invoices_clients_clist", conf_params->xdf_invoices_clients_clist);
	search(buff, "invoices_goods_clist", conf_params->xdf_invoices_goods_clist);
	search(buff, "prefix_nr_invoices", conf_params->xdf_prefix_nr_invoices);
	search(buff, "kind_of_nr_invoices", conf_params->xdf_kind_of_nr_invoices);
	search(buff, "info_text", conf_params->xdf_info_text);
	search(buff, "print_command", conf_params->xdf_print_command);
	search(buff, "print_format", conf_params->xdf_print_format);
	search(buff, "goods_prices", conf_params->xdf_goods_prices);
	search(buff, "ip", conf_params->xdf_ip);

}

/*
 * Funckja odczytuje parametry z command line i ustawia zmienn
 * conf_params nadpisujc jej wartoci. Parametry z cmdline maj wikszy
 * priorytet ni te z plikw.
 *
 */
int read_cmdopt(int argc, char **argv, configData *conf_params)
{
	int c ;
//	int digit_optind = 0;
	while (1)
	{
//		int this_option_optind = optind ? optind : 1;
		int option_index = 0;
		
		static struct option long_options[] = {
				  {"db_name",1,0,'d'},
				  {"db_host",1,0,'o'},
				  {"db_port",1,0,'p'},
				  {"db_user",1,0,'u'},
				  {"db_password",1,0,'s'},
				  {"sp_nazwa_firmy",1,0,'n'},
				  {"sp_wlasciciel",1,0,'w'},
				  {"sp_kod_pocztowy",1,0,'k'},
				  {"sp_miejscowosc",1,0,'m'},
				  {"sp_miejsce_wystawienia",1,0,'y'},
				  {"sp_telefon",1,0,'t'},
				  {"sp_fax",1,0,'f'},
				  {"sp_komorka",1,0,'c'},
				  {"sp_bank",1,0,'b'},
				  {"sp_nr_konta",1,0,'N'},
				  {"sp_ulica",1,0,'l'},
				  {"sp_nip",1,0,'i'},
				  {"sp_email",1,0,'e'},
				  {"sp_www",1,0,'W'},
				  {"xdf_invoices_clients_clist",1,0,'C'},
				  {"xdf_invoices_goods_clist",1,0,'G'},
				  {"xdf_prefix_nr_invoices",1,0,'P'},
				  {"xdf_kind_of_nr_invoices",1,0,'K'},
				  {"xdf_info_text",1,0,'T'},
				  {"xdf_print_command",1,0,'R'},
				  {"xdf_print_format",1,0,'F'},
				  {"xdf_goods_prices",1,0,'S'},
				  {"xdf_ip",1,0,'I'},
				  {"conf_path",1,0,'D'},
				  {"help",0, 0, 'h'},
				  {0, 0, 0, 0}
		};
		c = getopt_long (argc, argv, "d:o:p:u:s:n:w:k:m:y:t:f:c:b:N:l:i:e:W:C:G:P:K:T:R:F:S:I:D:h", long_options, &option_index);
		if (c == -1) break;

		switch (c) {
			case 'd':
				strncpy(conf_params->db_name, optarg, 256);
				break;
			case 'o':
				strncpy(conf_params->db_name, optarg, 256);
				break;
			case 'p':
				strncpy(conf_params->db_name, optarg, 256);
				break;
			case 'u':
				strncpy(conf_params->db_name, optarg, 256);
				break;
			case 's':
				strncpy(conf_params->db_name, optarg, 256);
				break;
			case 'n':
				strncpy(conf_params->sp_nazwa_firmy, optarg, 256);
				break;
			case 'w':
				strncpy(conf_params->sp_wlasciciel, optarg, 256);
				break;
			case 'k':
				strncpy(conf_params->sp_kod_pocztowy, optarg, 256);
				break;
			case 'm':
				strncpy(conf_params->sp_miejscowosc, optarg, 256);
				break;
			case 'y':
				strncpy(conf_params->sp_miejsce_wystawienia, optarg, 256);
				break;
			case 't':
				strncpy(conf_params->sp_telefon, optarg, 256);
				break;
			case 'f':
				strncpy(conf_params->sp_fax, optarg, 256);
				break;
			case 'c':
				strncpy(conf_params->sp_komorka, optarg, 256);
				break;
			case 'b':
				strncpy(conf_params->sp_bank, optarg, 256);
				break;
			case 'N':
				strncpy(conf_params->sp_nr_konta, optarg, 256);
				break;
			case 'l':
				strncpy(conf_params->sp_ulica, optarg, 256);
				break;
			case 'i':
				strncpy(conf_params->sp_nip, optarg, 256);
				break;
			case 'e':
				strncpy(conf_params->sp_email, optarg, 256);
				break;
			case 'W':
				strncpy(conf_params->sp_www, optarg, 256);
				break;
			case 'C':
				strncpy(conf_params->xdf_invoices_clients_clist, optarg, 256);
				break;
			case 'G':
				strncpy(conf_params->xdf_invoices_goods_clist, optarg, 256);
				break;
			case 'P':
				strncpy(conf_params->xdf_prefix_nr_invoices, optarg, 256);
				break;
			case 'K':
				strncpy(conf_params->xdf_kind_of_nr_invoices, optarg, 256);
				break;
			case 'T':
				strncpy(conf_params->xdf_info_text, optarg, 256);
				break;
			case 'R':
				strncpy(conf_params->xdf_print_command, optarg, 256);
				break;
			case 'F':
				strncpy(conf_params->xdf_print_format, optarg, 256);
				break;
			case 'S':
				strncpy(conf_params->xdf_goods_prices, optarg, 256);
				break;
			case 'I':
				strncpy(conf_params->xdf_ip, optarg, 256);
				break;
			case 'D':
				strncpy(conf_params->conf_path, optarg, 256);
				break;
			case 'h':
				show_xdef_usage();
				return 1;
				break;
		}
	}
	return 0;
}

void show_xdef_usage(void)
{
	printf("\n\nUsage: xdefactor-ng [OPTIONS]\n"
 			 "\t[--db_name | -d ]\t\tPostgreSQL: nazwa bazy danych\n"
 			 "\t[--db_host | -o ]\t\tPostgreSQL: host\n"
 			 "\t[--db_port | -p ]\t\tPostgreSQL: port\n"
 			 "\t[--db_user | -u ]\t\tPostgreSQL: nazwa uytkownika\n\n"
 			 "\t[--db_password | -s ]\t\tPostgreSQL: haso uytkownika\n\n"
 			 "\t[--sp_nazwa_firmy | -n ]\t\tSprzedawca: nazwa firmy\n"
 			 "\t[--sp_wlasciciel | -w ]\t\tSprzedawca: waciciel\n"
 			 "\t[--sp_kod_pocztowy | -k ]\t\tSprzedawca: kod pocztowy\n"
 			 "\t[--sp_miejscowosc | -m ]\t\tSprzedawca: miejscowosc\n"
 			 "\t[--sp_miejsce_wystawienia | -y ]\t\tSprzedawca: miejsce wystawienia faktury\n"
 			 "\t[--sp_telefon | -t ]\t\tSprzedawca: telefon\n"
 			 "\t[--sp_fax | -f ]\t\tSprzedawca: fax\n"
 			 "\t[--sp_komorka | -c ]\t\tSprzedawca: komorka\n"
 			 "\t[--sp_bank | -b ]\t\tSprzedawca: bank\n"
 			 "\t[--sp_nr_konta | -N ]\t\tSprzedawca: nr_konta\n"
 			 "\t[--sp_ulica | -l ]\t\tSprzedawca: ulica\n"
 			 "\t[--sp_nip | -i ]\t\tSprzedawca: nip\n"
 			 "\t[--sp_email | -e ]\t\tSprzedawca: email\n"
 			 "\t[--sp_www | -W ]\t\tSprzedawca: www\n\n"
			 "\t[--xdf_invoices_clients_clist | -C ]\t\tXdeFactor: invoices_clients_clist\n\n"
			 "\t[--xdf_invoices_goods_clist | -G ]\t\tXdeFactor: invoices_goods_clist\n\n"
			 "\t[--xdf_prefix_nr_invoices | -P ]\t\tXdeFactor: prefix_nr_invoices\n\n"
			 "\t[--xdf_kind_of_nr_invoices | -K ]\t\tXdeFactor: kind_of_nr_invoices\n\n"
			 "\t[--xdf_info_text | -T ]\t\tXdeFactor: info_text\n\n"
			 "\t[--xdf_print_command | -R ]\t\tXdeFactor: print_command\n\n"
			 "\t[--xdf_goods_prices | -F ]\t\tXdeFactor: goods_prices\n\n"
			 "\t[--xdf_ip | -S ]\t\tXdeFactor: ip\n\n"
			 "\t[--xdf_ | -I ]\t\tXdeFactor: \n\n"
			 "\t[--conf_path | -D ]\t\tSciezka do katalogu z plikami konfiguracyjnymi\n"
 			 "\t[--help | -h ]\t\tPokazuje t informacj\n\n"
			 "\tXdeFactor-ng jest programem do prowadzenia ksigowoci w maej firmie.\n"
			"\tWicej szczegowych informacji znajdziesz na stronie:\n"
			"\thttp://defactor-ng.gnu.pl/\n\n");
}

