/* ---------------------------------------------------------------------
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.  

   Copyright (c) 2001-2002 by Michal Ociepka <M.Ociepka@netsync.pl>

   For more info:
   http://xdefactor.netsync.pl
   --------------------------------------------------------------------- */

#include "callbacks.h"
#include "config.h"

void *checkHandle (void *function, const char *line)
{
	if (function==NULL) {	
		printf("\nNie mog zaadowa obiektu: %s\n. Bd: %s\n", line, dlerror());
		exit (EXIT_FAILURE);
	}
	return NULL;
}
void *checkErrMsg (const char *err_msg)
{
	if ((err_msg = dlerror ()) != NULL) {
		printf("\nNie mog odnale funkcji: %s\n", err_msg);
		exit (EXIT_FAILURE);
	}
	return NULL;
}				
void xdfDisconnect (xdfConnModule *conn_mod)
{
  conn_mod->kind_of_conn = 0;
  xdfConnSQL (conn_mod);
}


/**
 *   Funkcja do aktualizowania danych wyswietlanych w status barze.
 *   @param file_name    - wskaznik do status baru
 *
 *   @author Marcin Mazurek <M.Mazurek@netsync.pl>
 *   @version 1.0
 */

gboolean update_status_bar(GtkStatusbar *status_bar)
{
	time_t t;
	char buffer[256];
	
	time(&t);
	sprintf(buffer, "XdeFactor-ng v%s, %s", VERSION, ctime(&t));
	buffer[strlen(buffer)-1]='\0';
	gtk_statusbar_push (GTK_STATUSBAR(status_bar),
						 gtk_statusbar_get_context_id ( GTK_STATUSBAR(status_bar),
									"Glowny statusbar"), buffer);
	return TRUE;
}

configData	conf_params;


int main( int argc,
          char *argv[] )
{
	GtkWidget *window;
	GtkWidget *main_vbox;
	GtkWidget *menu[3];
	GtkWidget *root_menu[3];
	GtkWidget *menubar;
	GtkWidget *menu_items;  
	GtkWidget *status_bar;
	FILE *fp[2];
	GtkWidget *fixed_main; 
	char line[128];
	void *handle; 
	GString *string;
	int i;
	gchar	buff[256];

	void (*xdef_module)(xdfConnModule *conn_mod);
	const char *err_msg=NULL;
	xdfDialogMsgStruct xdfDialogMsgStruct_;
	xdfConnModule conn_mod;
 
	conn_mod.connect=-1;
	
	/* Sprawdzam opcje -D */
	if (argc>0)  read_conf_path(argc, argv, &conf_params);
	
	/* Czytam konfiguracje z pliku */
	read_fileopt(&conf_params);

	/* Czytamy pozostay parametry z cmdline o ile takie s. */
	if (argc>0 && read_cmdopt(argc, argv, &conf_params)!=0) return 1;

	gtk_init (&argc, &argv);
	gtk_set_locale();
	setlocale (LC_NUMERIC, "C");

	conn_mod.kind_of_conn=1;
	/* Nawiazywanie polaczenia z baza danych - PostgreSQL  */
	if(xdfConnSQL(&conn_mod) == 0){
		xdfDialogMsgStruct_.msg = g_string_new ("\nPołączenie z bazą danych zostało nawiązane\n\n\tXdeFactor-ng");
		xdfDialogMsgStruct_.title = g_string_new ("RAPORT POŁĄCZENIA Z BAZĄ DANYCH - XdeFactor-ng");
		xdfDialogMsgStruct_.type = GTK_MESSAGE_INFO;
  	}else{
		xdfDialogMsgStruct_.msg = g_string_new ("\nPołączenie z bazą danych NIE zostało nawiązane\n\n\tXdeFactor-ng");
		xdfDialogMsgStruct_.title = g_string_new ("RAPORT POŁĄCZENIA Z BAZĄ DANYCH - XdeFactor-ng");
		xdfDialogMsgStruct_.type = GTK_MESSAGE_WARNING;
	}

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_resize (GTK_WINDOW (window), 800, 600);
	g_signal_connect (G_OBJECT (window), "destroy", 
						 		G_CALLBACK (xdfDisconnect), &conn_mod);

	g_signal_connect (G_OBJECT (window), "destroy", 
							    G_CALLBACK (gtk_main_quit), NULL);

	gtk_window_set_title (GTK_WINDOW (window), TITLE);
	
  /*
	* Do tej funkcji potrzebne jest gtk 2.2, ktrego np. nie ma jeszcze
	* w Debianie Stable. Trzeba pokombinowa nad podmian.
	*
  gtk_window_set_default_icon_from_file( "/usr/share/xdefactor-ng/images/icon32x32.xpm", 
														NULL);
	*/													
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);

	fixed_main = gtk_fixed_new ();
 /* moduleAuthors (fixed_main);*/
  for(i=1; i != 4; i++){
    menu[i] = gtk_menu_new ();
  }
  root_menu[1] = gtk_menu_item_new_with_label (MENU_1);
  root_menu[2] = gtk_menu_item_new_with_label (MENU_2);
  root_menu[3] = gtk_menu_item_new_with_label (MENU_3);

	for(i=1; i != 4; i++){
   	gtk_widget_show (root_menu[i]);
	}

	string = g_string_new (conf_params.conf_path);

	sprintf(buff, "%s/%s", conf_params.conf_path, MOD_CONF);
	if((fp[1] = fopen (buff, "r")) < 0){
		perror (buff);
		exit(EXIT_FAILURE);
	}

  while ((fgets (line, 127, fp[1])) != NULL){
    if(line[0] != '#'){
/* tu uzyskujemy plik opisujacy modul */
      line[ strlen(line)-1] = '\0'; 
/* wyznaczam kompletna sciezke opisujaca modul */
      g_string_append (string, line); 

/* szukaj miejsce w jakim menu wrzucic modul */
      search (string->str, "submenu", line);   
      menu_items = gtk_menu_item_new_with_label(line);
      search (string->str, "menu", line);   
/* sprawdzam gdzie umiescic modul */
      if(strcmp (line, MENU_1) == 0)
	gtk_menu_shell_append (GTK_MENU_SHELL (menu[1]), menu_items);
      if(strcmp (line, MENU_2) == 0)
	gtk_menu_shell_append (GTK_MENU_SHELL (menu[2]), menu_items);
      if(strcmp (line, MENU_3) == 0)
	gtk_menu_shell_append (GTK_MENU_SHELL (menu[3]), menu_items);

      fgets (line, 127, fp[1]);
      line[ strlen(line)-1] = '\0'; /* nazwa modulu */
/* laduje odpowiedni obiekt */
      handle = dlopen (line, RTLD_LAZY); 
			checkHandle (handle, line);
      dlerror ();
/* sprawdzam i ustalam jaka funkcje ma modul */
      search (string->str, "name", line);
      xdef_module = dlsym(handle, line);
			
			checkErrMsg (err_msg);

/* Wywolujemy funkcje, ktora pokaze nam modul */
      g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
				G_CALLBACK (xdef_module),
				&conn_mod);

/* wyrzucam sciezke dla modulu poprzedniego */
      g_string_erase (string, strlen(conf_params.conf_path), -1);
      gtk_widget_show (menu_items);
    }
  }
	//otwieram plik do logowania
	if((conn_mod.fd=fopen (LOG_FILE, LOG_ATR))==NULL){
		perror ("Nie mog otworzy pliku: " LOG_FILE);
		exit (EXIT_FAILURE);
	}
	xdfLog (conn_mod.fd, "Start");


  for(i=1; i != 4; i++){
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu[i]), menu[i]);
  }
	/* Konczymy aplikacje */
  menu_items = gtk_menu_item_new_with_label("Koniec");
	gtk_menu_shell_append (GTK_MENU_SHELL (menu[1]), menu_items);

  /* Wywolujemy funkcje, ktora pokaze nam modul */
  g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
	G_CALLBACK (gtk_main_quit),
	NULL);

/* wyrzucam sciezke dla modulu poprzedniego */
  g_string_erase (string, strlen(conf_params.conf_path), -1);
  gtk_widget_show (menu_items);

  main_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), main_vbox);
  gtk_widget_show (main_vbox);
 
  menubar = gtk_menu_bar_new (); 
  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 1);
  gtk_widget_show (menubar);

  gtk_box_pack_start (GTK_BOX (main_vbox), fixed_main, FALSE, TRUE, 0);

  for(i=1; i != 4; i++){
    gtk_menu_shell_append (GTK_MENU_SHELL (menubar), root_menu[i]);
  }
	
	/* Dodaj status_bar dla gwnego okna aplikacji. */
	status_bar = gtk_statusbar_new();
	gtk_box_pack_end (GTK_BOX (main_vbox), status_bar, FALSE, TRUE,  0);
	gtk_widget_show (status_bar);
	gtk_statusbar_push (GTK_STATUSBAR(status_bar),
						 gtk_statusbar_get_context_id ( GTK_STATUSBAR(status_bar),
									"Glowny statusbar"), "XdeFactor-ng");
	gtk_timeout_add(50, (GtkFunction) update_status_bar, GTK_STATUSBAR(status_bar)); 

  gtk_widget_show (fixed_main);
  gtk_widget_show (window);
  xdfDialogMsg (&xdfDialogMsgStruct_);

  gtk_main ();

  g_string_free (string, TRUE);

  return 0;
}
