#include "../defs.h"

/* Funkcja sprawdza czy faktura o danym numerze istnieje. */
gboolean invoiceNumberExists(xdfInvConnModule *conn_mod, gint number, gint year)
{
	GString *query;
	PGresult *res;

	query = g_string_new("");
	g_string_printf(query,
		"SELECT number "
		"FROM invoices "
		"WHERE is_active=true "
		"AND extract(year from sell_date)=%d "
		"AND number=%d;",
		year,
		number);
	res = xdfPQexec(conn_mod->conn, query->str);
	g_string_free(query, TRUE);
	if (PQresultStatus(res) == PGRES_TUPLES_OK && PQntuples(res) > 0) {
		PQclear(res);
		return TRUE;
		}
	PQclear(res);
	return FALSE;
}

/* Funkcja zwraca następny numer faktury. */
gint getNextInvoiceNumber(xdfInvConnModule *conn_mod)
{
	PGresult *res;
	gint i = 0;
	gchar *result;
	res = xdfPQexec(conn_mod->conn,
		"SELECT max(number),"
		"extract(year from sell_date) AS rok "
		"FROM invoices "
		"WHERE is_active=true "
		"AND extract(year from sell_date)=extract(year from current_date) "
		"GROUP BY rok;");
	if (PQresultStatus(res) == PGRES_TUPLES_OK && PQntuples(res) == 1) {
		result = PQgetvalue(res, 0, 0);
		i = atoi(result);
		}
	PQclear(res);
	return i + 1;
}

/* Funkcja ustawia numer faktury w invoiceNr.
   Zwraca ustawiony numer faktury. */
gint setInvoiceNumber(xdfInvConnModule *conn_mod, GString *invoiceNr)
{
	gint nextInvNr;
	time_t curtime;
	struct tm *loctime;
	gchar buff[256];

	curtime = time (NULL);
	loctime = localtime (&curtime);
	nextInvNr = getNextInvoiceNumber(conn_mod);
	if (!strcmp(conf_params.xdf_kind_of_nr_invoices, "NR/MM/YYYY"))
		sprintf(buff, "%d/%02d/%04d", nextInvNr,
			loctime->tm_mon + 1, loctime->tm_year + 1900);
	else
		if (!strcmp(conf_params.xdf_kind_of_nr_invoices, "NR/YYYY"))
			sprintf(buff, "%d/%04d", nextInvNr,
				loctime->tm_year + 1900);
		else
			if (!strcmp(conf_params.xdf_kind_of_nr_invoices, "YYYY/NR"))
				sprintf(buff, "%04d/%d", loctime->tm_year + 1900,
					nextInvNr);
	g_string_assign(invoiceNr, buff);
	return nextInvNr;
}

/* Funkcja sprawdza poprawno?? numeru faktury w formularzu.
   Je?li numer faktury jest poprawny zwraca numer faktury.
   Je?li numer faktury jest niepoprawny to zwraca zero.
   o ile checkNumberIfExists jest TRUE.
   Je?li numer faktury jest poprawny, ju? jest w bazie faktur oraz
   checkNumberIfExists jest TRUE funkcja umieszcza
   nast?pny numer faktury w invoiceNr i go zwraca. */
gint validateInvoiceNumber(xdfInvConnModule *conn_mod, GString *invoiceNr,
	gboolean checkNumberIfExists, gint *year)
{
	PGresult *res;
	GString *query;
	gint count, nr = 0, month, type = 0;
	gchar *nrend, *monthend, *yearend;
	gchar **tokens = g_strsplit(invoiceNr->str, "/", 4);

	for (count = 0; tokens[count]; count++);
	if (count == 3 &&
		!strcmp(conf_params.xdf_kind_of_nr_invoices, "NR/MM/YYYY"))
		{
		nr = strtol(tokens[0], &nrend, 10);
		month = strtol(tokens[1], &monthend, 10);
		*year = strtol(tokens[2], &yearend, 10);
		if (!strlen(nrend) && !strlen(monthend) && !strlen(yearend))
			type = 1;
		}
	else
		if (count == 2)
			{
			if (!strcmp(conf_params.xdf_kind_of_nr_invoices, "NR/YYYY"))
				{
				nr = strtol(tokens[0], &nrend, 10);
				*year = strtol(tokens[1], &yearend, 10);
				if (!strlen(nrend) && !strlen(yearend))
					type = 2;
				}
			else
				{
				*year = strtol(tokens[0], &yearend, 10);
				nr = strtol(tokens[1], &nrend, 10);
				if (!strlen(nrend) && !strlen(yearend))
					type = 3;
				}
			}
	g_strfreev(tokens);
	
	if (!type)
		return 0;
	
	query = g_string_new("");
	g_string_printf(query,
		"SELECT number "
		"from invoices "
		"where is_active=true "
		"and extract(year from sell_date)=%d "
		"and number=%d;",
		*year,
		nr);
	res = xdfPQexec(conn_mod->conn, query->str);
	g_string_free(query, TRUE);
	
	if (PQresultStatus(res) == PGRES_TUPLES_OK && PQntuples(res) > 0
		&& checkNumberIfExists) {
		PQclear(res);
		return setInvoiceNumber(conn_mod, invoiceNr);
		}	
	PQclear(res);
	return nr;
}
